# -*- coding: utf-8 -*-
# Generated by Django 1.9.12 on 2017-03-17 14:02


import django.utils.timezone
from django.db import migrations, models

import model_utils.fields


class Migration(migrations.Migration):

    dependencies = [
        ('sap_success_factors', '0002_auto_20170224_1545'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='catalogtransmissionaudit',
            name='end',
        ),
        migrations.RemoveField(
            model_name='catalogtransmissionaudit',
            name='start',
        ),
        migrations.AddField(
            model_name='catalogtransmissionaudit',
            name='created',
            field=model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created'),
        ),
        migrations.AddField(
            model_name='catalogtransmissionaudit',
            name='modified',
            field=model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified'),
        ),
        migrations.AddField(
            model_name='historicalsapsuccessfactorsenterprisecustomerconfiguration',
            name='user_type',
            field=models.CharField(choices=[('user', 'User'), ('admin', 'Admin')], default='user', max_length=20, verbose_name='SAP User Type'),
        ),
        migrations.AddField(
            model_name='sapsuccessfactorsenterprisecustomerconfiguration',
            name='user_type',
            field=models.CharField(choices=[('user', 'User'), ('admin', 'Admin')], default='user', max_length=20, verbose_name='SAP User Type'),
        ),
    ]
