# -*- coding: utf-8 -*-
# Generated by Django 1.10.2 on 2016-11-14 04:34


from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('enterprise', '0003_auto_20161104_0937'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprisecustomer',
            name='identity_provider',
            field=models.SlugField(blank=True, default=None, null=True),
        ),
        migrations.AddField(
            model_name='historicalenterprisecustomer',
            name='identity_provider',
            field=models.SlugField(blank=True, default=None, null=True),
        ),
    ]
